/// <reference path="./eServiceFormHelpers.js"/>

/**
 * Adds a Validator that ensures a person doesn't insert a SIN Number into a Field
 * @param {string} fieldName The fieldname to secure
 * @param {string} fieldLabel The Label of the field
 */
function addSinCheckValidator(fieldName, fieldLabel) {
   var evaluationfunction = function () {
      const field = document.getElementById(fieldName);
      let value = field.value
      console.log("Checking for SIN: " + value);
      let hasSin = SINNumber.ContainsSin(value);
      let sinMatches = SINNumber.GetSinMatch(value);
      if (hasSin) {
         if (SINNumber != undefined && SINNumber != null && sinMatches != undefined && sinMatches != null) {
            for (let i = 0; i < sinMatches.length; i++) {
               const sin = sinMatches[i];
               if (sin != undefined) {
                  const validSin = SINNumber.Validate(sin);
                  if (validSin) {
                     return false;
                  }
               }
            }
            return true;
         }
      }
      return !hasSin;
   };
   if (isNullOrWhitespace(fieldLabel)) {
      fieldLabel = getLabelText(fieldName);
   }
   let errorMsg;
   if (isEnglish) { errorMsg = "cannot contain SIN's."; }
   else { errorMsg = "ne peut pas contenir de NAS."; }
   createValidator(fieldName, fieldLabel, "SINFieldValidator", null, false, errorMsg, evaluationfunction);
}

/**
 * Configures the Service Request Form with all the required validation and Views
 * @param {{}} options Optional Parameters to control the Loading of the Form
 * @returns {Object.<string, InitializeLogic>} The Service Request Initialize Logic
 */
function CreateServiceRequestInitializationLogic(options) {
   if (options == null || options == undefined) { options = {}; }

   let initalizeLogic = {};

   //#region Begin Initialization
   createInitializeLogic(initalizeLogic, "esdc_languageofservice", function () {
      $("#esdc_languageofservice").change(function () {
         let esdc_languageofservice = $(this).val();
         if (esdc_languageofservice == "564190002") {
            $("#esdc_videorelayservicevrstelephonenumber").parents("td").hide();
            $("#esdc_teletypewriterttytelephone").parents("td").hide();
            $("#esdc_otherlanguage").parents("td").show();
            addRequiredValidator("esdc_otherlanguage", null, null);
            configureRequiredLabels({
               fields: ["esdc_otherlanguage"]
            });
         }else if(esdc_languageofservice == "100000000" || esdc_languageofservice == "100000001"){
            $("#esdc_otherlanguage").parents("td").hide();
            removeRequiredValidator("esdc_otherlanguage");
            $("#esdc_videorelayservicevrstelephonenumber").parents("td").show();
            $("#esdc_teletypewriterttytelephone").parents("td").show();
         }
         else {
            $("#esdc_otherlanguage").parents("td").hide();
            $("#esdc_videorelayservicevrstelephonenumber").parents("td").hide();
            $("#esdc_teletypewriterttytelephone").parents("td").hide();

            removeRequiredValidator("esdc_otherlanguage");

         }
      });
      $("#esdc_languageofservice").change();
   });

   createInitializeLogic(initalizeLogic, "esdc_otherlanguage", function () {
      addSinCheckValidator("esdc_otherlanguage", null);
      $(this.jquery).removeAttr("aria-label");
   });
   createInitializeLogic(initalizeLogic, "esdc_firstname", function () {
      $("#esdc_firstname").attr("autocomplete", "given-name");
      addSinCheckValidator("esdc_firstname", null);
      $(this.jquery).removeAttr("aria-label");
   });
   createInitializeLogic(initalizeLogic, "esdc_lastname", function () {
      $("#esdc_lastname").attr("autocomplete", "family-name");
      addSinCheckValidator("esdc_lastname", null);
      $(this.jquery).removeAttr("aria-label");
   });
   createInitializeLogic(initalizeLogic, "esdc_accessibilityinformation", function () {
      addSinCheckValidator("esdc_accessibilityinformation", null);
      $(this.jquery).removeAttr("aria-label");
   });
   createInitializeLogic(initalizeLogic, "esdc_additionalinformation", function () {
      addSinCheckValidator("esdc_additionalinformation", null);
      $(this.jquery).removeAttr("aria-label");
   });

   createInitializeLogic(initalizeLogic, "esdc_locationcountryid", async function () {
      // Set the Loading text before we load the data
      let loadingText = getTranslatedText("Loading...", "Chargement en cours...");
      let defaultOption = $("#esdc_locationcountryid").children().first();
      let defaultOptionOriginalLabel = defaultOption.attr("label");
      let defaultOptionOriginalAria = defaultOption.attr("aria-label");

      defaultOption.attr("label", loadingText);
      defaultOption.attr("aria-label", loadingText);

      // Retrieve and Parse the Address data 
      let addressMetadata = await loadAddressMetadata(null);
      let countries = addressMetadata.countries.value.filter(x => {
         if (x.esdc_allowedforservicerequest == true) {
            return true;
         }
         return false;
      }).map(x => {
         return {
            id: x.esdc_countryid,
            english: x.esdc_nameenglish,
            french: x.esdc_namefrench,
            text: getTranslatedText(x.esdc_nameenglish, x.esdc_namefrench),
            value: x.esdc_countryid,
            groupId: x.esdc_groupkey,
            record: x,
            attributes: [
               { key: "data-name-english", value: x.esdc_nameenglish },
               { key: "data-name-french", value: x.esdc_namefrench },
               { key: "data-countryid", value: x.esdc_countryid }
            ],
         }
      });

      translateLookupNameField("esdc_locationcountryid", {
         idField: "esdc_locationcountryid",
         nameField: "esdc_locationcountryid option",
         list: countries
      });

      // Remove the Loading Text
      defaultOption.attr("label", defaultOptionOriginalLabel);
      defaultOption.attr("aria-label", defaultOptionOriginalAria);

      // Set Canada as Default
      let existingLookup = getLookupField("esdc_locationcountryid");
      if (existingLookup == null || existingLookup.id == "-1") {
         setLookupField("esdc_locationcountryid", {
            id: "0cf5389e-97ae-eb11-8236-000d3af4bfc3",
            name: "Canada",
            entityName: "esdc_country"
         });
      }

      $("#esdc_locationcountryid").change(function () {
         let selectedValue = $(this).val();
         // if (isNullOrWhitespace(selectedValue)) { return; }

         if (selectedValue == "0cf5389e-97ae-eb11-8236-000d3af4bfc3") /* Canada */ {
            $("#esdc_locationprovinceterritoryid").parents("td").show();
            $("#esdc_postalzipcode").parents("td").show();
            
            configureRequiredLabels({
               fields: ["esdc_locationprovinceterritoryid"]
            });
         }
         else if (selectedValue == "d09b7e05-0952-ec11-8f8e-000d3a0a0f99") /* Outside of Canada */ {
            setLookupField("esdc_locationprovinceterritoryid", null);
            $("#esdc_locationprovinceterritoryid").parents("td").hide();

            $("#esdc_postalzipcode").val("");
            $("#esdc_postalzipcode").parents("td").hide();
         }
         // else /* Every other country */ {
         //    setLookupField("esdc_locationprovinceterritoryid", null);
         //    $("#esdc_locationprovinceterritoryid").parents("td").hide();
         // }
      });
      $("#esdc_locationcountryid").change();
   });
   createInitializeLogic(initalizeLogic, "esdc_locationprovinceterritoryid", async function () {
      // Set the Loading text before we load the data
      let loadingText = getTranslatedText("Loading...", "Chargement en cours...");
      let defaultOption = $("#esdc_locationprovinceterritoryid").children().first();
      let defaultOptionOriginalLabel = defaultOption.attr("label");
      let defaultOptionOriginalAria = defaultOption.attr("aria-label");

      defaultOption.attr("label", loadingText);
      defaultOption.attr("aria-label", loadingText);

      // Retrieve and Parse the Address data 
      let addressMetadata = await loadAddressMetadata(null);
      let provinces = addressMetadata.provinceTerritoryStates.value.filter(x => {
         /* Search for Canadian Provinces Canada */
         if (x._esdc_countryid_value == "0cf5389e-97ae-eb11-8236-000d3af4bfc3") {
            return true;
         }
         return false;
      }).map(x => {
         return {
            id: x.esdc_provinceterritorystateid,
            english: x.esdc_nameenglish,
            french: x.esdc_namefrench,
            text: getTranslatedText(x.esdc_nameenglish, x.esdc_namefrench),
            value: x.esdc_provinceterritorystateid,
            groupId: x.esdc_groupkey,
            record: x,
            attributes: [
               { key: "data-name-english", value: x.esdc_nameenglish },
               { key: "data-name-french", value: x.esdc_namefrench },
               { key: "data-countryid", value: x._esdc_countryid_value }
            ]
         }
      });

      provinces.push({
         id: "",
         english: "Select your Province/Territory",
         french: "Sélectionnez votre province/territoire",
         text: getTranslatedText("Select your Province/Territory", "Sélectionnez votre province/territoire"),
         value: ""
      });

      translateLookupNameField("esdc_locationprovinceterritoryid", {
         idField: "esdc_locationprovinceterritoryid",
         nameField: "esdc_locationprovinceterritoryid option",
         list: provinces,
         noMatchFoundMode: "remove"
      });

      // Remove the Loading Text
      defaultOption.attr("label", defaultOptionOriginalLabel);
      defaultOption.attr("aria-label", defaultOptionOriginalAria);

      // Configure the Minimum Validator
      removeRequiredValidator("esdc_locationprovinceterritoryid");
      createValidator("esdc_locationprovinceterritoryid", null, "LookupFieldSelection", "esdc_locationprovinceterritoryid", true, getTranslatedText(
         "is a required field.",
         "est un champ obligatoire."
      ), function () {
         let esdc_locationcountryid_value = $("#esdc_locationcountryid").val();
         if (esdc_locationcountryid_value == "d09b7e05-0952-ec11-8f8e-000d3a0a0f99") /* Outside of Canada */ {
            setLookupField("esdc_locationprovinceterritoryid", null);
            return true;
         }

         let selectedValue = $("#esdc_locationprovinceterritoryid").val();
         if (isNullOrWhitespace(selectedValue) || selectedValue == "-1") {
            return false;
         }
         return true;
      });

      configureRequiredLabels({
         fields: ["esdc_locationprovinceterritoryid"]
      });
   });
   createInitializeLogic(initalizeLogic, "esdc_postalzipcode", function () {
      this.jquery.change(function () {
         let selectedValue = $(this).val()?.toUpperCase();
         $(this).val(selectedValue);
      });
   });

   createInitializeLogic(initalizeLogic, "esdc_telephonenumber", function () {
      $("#esdc_telephonenumber").attr("autocomplete", "tel");
      $("#esdc_telephonenumber").attr("type", "tel");
      $(this.jquery).removeAttr("aria-label");
   });

   createInitializeLogic(initalizeLogic, "emailaddress", function () {
      configureRecommendedLabels({
         fields: ["emailaddress"]
      });

      $("#emailaddress").attr("autocomplete", "email");
      $(this.jquery).removeAttr("aria-label");

      // Hide by default
      $("#emailaddress").parents("td").hide();
   });

   createInitializeLogic(initalizeLogic, "esdc_browseragent", function () {
      SaveUserAgent("esdc_browseragent");
      // $("#esdc_browseragent").attr("readonly", "readonly");
   });
   createInitializeLogic(initalizeLogic, "esdc_usertimezone", function () {
      try {
         var timezone = Intl.DateTimeFormat().resolvedOptions().timeZone;
         $("#esdc_usertimezone").val(timezone);
      } catch (ex) { }
   });
   createInitializeLogic(initalizeLogic, "esdc_applicationsubmittedon", function () {
      createValidator("esdc_applicationsubmittedon", null, "UpdateSubmittedOn", "esdc_applicationsubmittedon", false, "", function () {
         try {
            setDateTimeField("esdc_applicationsubmittedon", moment.utc().toDate(), true);
         } catch (ex) { }
         return true;
      });
   });
   createInitializeLogic(initalizeLogic, "esdc_clientdataentrycomplete", function () {
      createValidator("esdc_clientdataentrycomplete", null, "RequireYes", "esdc_clientdataentrycomplete", true, "",
         function () {
            setDateTimeField("esdc_applicationsubmittedon", moment.utc().toDate(), true);
            $("#esdc_clientdataentrycomplete").val("1");

            return true;
         }
      );
   });
   //#endregion

   return initalizeLogic;

}

async function createSRQuestionsSection() {
   let questions = await loadMetadata("/_api/esdc_servicerequestquestions?$filter=statecode eq 0", "esdc_servicerequestquestions", null, null);
   let groupings = await loadMetadata("/_api/esdc_portallistgroupings?$filter=statecode eq 0", "esdc_portallistgroupings", null, null);
   //console.log("Query has returned" + questions.value);

   let categoryQuestionPair = [];
   for (let y = 0; y < groupings.value.length; y++) {
      let group = groupings.value[y];
      if (group.esdc_displayinsrportal == true) {
         categoryQuestionPair.push({ group: group, questions: [] });
      }
   }

   for (let i = 0; i < questions.value.length; i++) {     
      let question = questions.value[i];
      let group = groupings.value.find(element => {
         if (question._esdc_portallistgroupingid_value == element.esdc_portallistgroupingid) {
            return true;
         }
         return false;
      });

      if (!group) { continue; } 

      let existingPair = categoryQuestionPair.find(element => {
         if (element.group.esdc_portallistgroupingid == group.esdc_portallistgroupingid) {
            return true;
         }
         return false;
      });

      if (existingPair != null) {
         existingPair.questions.push(question);
      }
      else {
         categoryQuestionPair.push({ group: group, questions: [question] });
      }
   }

   //Sort the array based on the Portal Group index 
   categoryQuestionPair.sort(function (a, b) {
      return a.group.esdc_index - b.group.esdc_index;
   });

   console.log("# Categories on Portal " + categoryQuestionPair.length);
   //categoryQuestionPair.questions.sort(categoryQuestionPair.questions.esdc_index);

   let containerElement = $('table[data-name="step_1_tab_section_5"] tbody tr:first td:first');
   for (let y = 0; y < categoryQuestionPair.length; y++) {
      let cqPair = categoryQuestionPair[y];

      var serviceOffering = document.createElement("fieldset");
      serviceOffering.setAttribute("name", getTranslatedText(cqPair.group.esdc_nameenglish, cqPair.group.esdc_namefrench));
      containerElement.append(serviceOffering);

      let groupTitle = document.createElement("legend");
      groupTitle.setAttribute("title", getTranslatedText(cqPair.group.esdc_nameenglish, cqPair.group.esdc_namefrench));
      serviceOffering.append(groupTitle);

      // Top tooltip
      if (cqPair.group.toptooltipenable == true) {
         let tooltipTop = document.createElement("div");
         if (!isNullOrWhitespace(cqPair.group.esdc_toptooltipenglish) || !isNullOrWhitespace(cqPair.group.esdc_toptooltipfrench)) {
            tooltipTop.innerHTML = getTranslatedText(cqPair.group.esdc_toptooltipenglish, cqPair.group.esdc_toptooltipfrench);
            serviceOffering.append(tooltipTop);
         }
      }

      // Render table
      let table = document.createElement("table");
      serviceOffering.append(table);

      // Render bottom tooltip
      if (cqPair.group.esdc_bottomtooltipenable == true) {
         let tooltipBottom = document.createElement("div");
         if (!isNullOrWhitespace(cqPair.group.esdc_bottomtooltipenglish) || !isNullOrWhitespace(cqPair.group.esdc_bottomtooltipfrench)) {
            tooltipBottom.innerHTML = getTranslatedText(cqPair.group.esdc_bottomtooltipenglish, cqPair.group.esdc_bottomtooltipfrench);
            serviceOffering.append(tooltipBottom);
         }
      }

      let tbdy = document.createElement("tbody");
      table.appendChild(tbdy);

      // Sort the questions and begin generation
      categoryQuestionPair[y].questions.sort(function (a, b) {
         return a.esdc_index - b.esdc_index;
      });

      for (let iQuestion = 0; iQuestion < categoryQuestionPair[y].questions.length; iQuestion++) {
         let question = cqPair.questions[iQuestion];

         let tr = document.createElement('tr');
         tbdy.appendChild(tr);

         let td = document.createElement('td');
         td.setAttribute("colspan", "1");
         td.setAttribute("rowspan", "1");
         $(td).attr("class", "clearfix cell checkbox-cell");
         tr.appendChild(td);

         // Create the tooltip metadata
         let fieldTooltips = [];

         // Configure the Top Tooltip
         if (question.esdc_toptooltipenabled == true) {
            if (!isNullOrWhitespace(question.esdc_toptooltipenglish) || !isNullOrWhitespace(question.esdc_toptooltipfrench)) {
               let tooltipTop = {
                  id: "SRquestion" + y + iQuestion + "_tooltip_top",
                  elementType: "div",
                  position: "above",
                  innerHtml: getTranslatedText(question.esdc_toptooltipenglish, question.esdc_toptooltipfrench),
                  isTooltip: true,
                  descriptionType: "tooltip"
               };
               fieldTooltips.push(tooltipTop);
            }
         }

         // Configure the Bottom Tooltip
         if (question.esdc_bottomtooltipenabled == true) {
            if (!isNullOrWhitespace(question.esdc_bottomtooltipenglish) || !isNullOrWhitespace(question.esdc_bottomtooltipfrench)) {
               let tooltipBottom = {
                  id: "SRquestion" + y + iQuestion + "_tooltip_bottom",
                  elementType: "div",
                  position: "below",
                  innerHtml: getTranslatedText(question.esdc_bottomtooltipenglish, question.esdc_bottomtooltipfrench),
                  isTooltip: true,
                  descriptionType: "tooltip"
               };
               fieldTooltips.push(tooltipBottom);
            }
         }

         // Create the Questions
         let soQuestion = createPortalField("SRquestion" + y + iQuestion, { labelText: getTranslatedText(question.esdc_nameenglish, question.esdc_namefrench) }, false, {
            elementType: "input",
            attributes: [
               { key: "class", value: "" },
               { key: "type", value: "checkbox" },
               { key: "data-questionid", value: question.esdc_servicerequestquestionid },
               { key: "data-categoryid", value: cqPair.group.esdc_portallistgroupingid },
               { key: "data-eservicerequest-checkbox-n2n", value: true }
            ],
            description: fieldTooltips
         });
         td.appendChild(soQuestion);
      }

      createCollapsable(serviceOffering, groupTitle.title, {
         defaultState: "closed",
         addBreakAfterDetails: true,
         containerType: "div"
      });
   }
}

async function createServiceRequestMappingsAPI() {
   console.log("createServiceRequestMappingsAPI")

   //Obtain the record information to associate to the question
   let recordInfo = getRecordInformation();

   // Log all the requests
   let allAPIRequestPromises = [];
   let allAPIRequestData = [];

   // Old Case Intake Error
   // let error;
   // if (isEnglish) { error = 'Please select at least one of the checkboxes under "How can we help you?"'; }
   // else { error = "Veuillez cocher au moins une des cases sous « Comment pouvons-nous vous aider? »"; }

   // Configure the validation on the collapsible
   createValidator("esdc_collapsibleServiceOfferings", "", "", null, false, getTranslatedText(
      " Please select at least one of the checkboxes under 'How can we help you?'",
      " Veuillez cocher au moins une des cases sous « Comment pouvons-nous vous aider? »"
   ), function () {
      let questionChecked = $("input[type=checkbox]:checked");
      if (questionChecked.length == 0) {
         return false;
      }
      return true;
   });

   //Obtain all the questions that have been checked
   let questionChecked = $("input[type=checkbox]:checked");
   for (let i = 0; i < questionChecked.length; i++) {
      let question = questionChecked[i];

      // Get te Question Id
      let srID = $(question).attr("data-questionid");
      if (isNullOrWhitespace(srID)) { console.log(`srID: ${srID} is invalid. Skipping association`); continue; }
      let apiData = {
         "esdc_ServiceRequestQuestionid@odata.bind": "/esdc_servicerequestquestions({0})".replace("{0}", srID),
         "esdc_ServiceRequestid@odata.bind": "/esdc_servicerequests({0})".replace("{0}", recordInfo.id)
      };
      allAPIRequestData.push(apiData);

      try {
         console.log("Attempting Create Record: esdc_servicerequestquestionmaps with data: " + JSON.stringify(apiData));
         let requestPromise = safeAjax({
            type: "POST",
            url: "/_api/esdc_servicerequestquestionmaps",
            contentType: "application/json",
            data: JSON.stringify(apiData),
            success: function (res, status, xhr) {
               let recordId = xhr.getResponseHeader("entityid");
               console.log("Record Created: esdc_servicerequestquestionmaps" + recordId);
            },
            error: function (XMLHttpRequest, textStatus, errorThrown) {
               console.log("There was an error");
               console.log(errorThrown);
            }
         });

         allAPIRequestPromises.push(requestPromise);
      }
      catch (ex) {
         console.log("An exception occured creating Mapping record");
         console.log(ex);
      }
   }

   // Log all the API Requests to the JSON field
   $("#esdc_questionmappingjson").val(JSON.stringify(allAPIRequestData));

   // Await all of the promises to be completed
   return await Promise.allSettled(allAPIRequestPromises);
}